/*
 *
 * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 *
 */

import fp from 'fastify-plugin'

// This plugin is used to log requests and responses.
export default fp(async (fastify) => {
  // Logs the request body with the 'trace' level
  fastify.addHook('preHandler', function (req, reply, done) {
    if (req.body) req.log.trace({ body: req.body }, 'parsed body')
    done()
  })

  fastify.addHook('onError', (req, reply, { message, stack, statusCode }, done) => {
    const error = { statusCode, stack }
    reply.log.error(error, message || 'request errored')
    done()
  })

  // Logs the request with the 'debug' level and also logs headers with the 'trace' level
  fastify.addHook('onResponse', (req, reply, done) => {
    const loggingOptions = { url: req.raw.url, res: reply, responseTime: reply.getResponseTime() }
    if (process.env.LOG_LEVEL === 'trace') loggingOptions.headers = req.headers
    reply.log.debug(loggingOptions, 'request completed')
    done()
  })
})
